@@ -20,12 +20,15 @@ from hyperliquid.utils.signing import (
     get_timestamp_ms,
     order_request_to_order_wire,
     order_wires_to_order_action,
+    sign_approve_builder_fee,
     sign_l1_action,
+    sign_usd_class_transfer_action,
     sign_usd_transfer_action,
+    sign_spot_transfer_action,
     sign_withdraw_from_bridge_action,
     sign_agent,
 )
-from hyperliquid.utils.types import Any, List, Meta, SpotMeta, Optional, Tuple, Cloid
+from hyperliquid.utils.types import Any, List, Meta, SpotMeta, Optional, Tuple, Cloid, BuilderInfo
 
 
 class Exchange(API):
@@ -45,46 +48,32 @@ class Exchange(API):
         self.wallet = wallet
         self.vault_address = vault_address
         self.account_address = account_address
-        self.info = Info(base_url, skip_ws=True)
-        if meta is None:
-            self.meta = self.info.meta()
-        else:
-            self.meta = meta
-
-        if spot_meta is None:
-            self.spot_meta = self.info.spot_meta()
-        else:
-            self.spot_meta = spot_meta
-
-        self.coin_to_asset = {asset_info["name"]: asset for (asset, asset_info) in enumerate(self.meta["universe"])}
-
-        # spot assets start at 10000
-        for spot_info in self.spot_meta["universe"]:
-            self.coin_to_asset[spot_info["name"]] = spot_info["index"] + 10000
+        self.info = Info(base_url, True, meta, spot_meta)
 
     def _post_action(self, action, signature, nonce):
         payload = {
             "action": action,
             "nonce": nonce,
             "signature": signature,
-            "vaultAddress": self.vault_address,
+            "vaultAddress": self.vault_address if action["type"] != "usdClassTransfer" else None,
         }
         logging.debug(payload)
         return self.post("/exchange", payload)
 
     def _slippage_price(
         self,
-        coin: str,
+        name: str,
         is_buy: bool,
         slippage: float,
         px: Optional[float] = None,
     ) -> float:
+        coin = self.info.name_to_coin[name]
         if not px:
             # Get midprice
             px = float(self.info.all_mids()[coin])
 
         # spot assets start at 10000
-        is_spot = self.coin_to_asset[coin] >= 10_000
+        is_spot = self.info.coin_to_asset[coin] >= 10_000
 
         # Calculate Slippage
         px *= (1 + slippage) if is_buy else (1 - slippage)
@@ -93,16 +82,17 @@ class Exchange(API):
 
     def order(
         self,
-        coin: str,
+        name: str,
         is_buy: bool,
         sz: float,
         limit_px: float,
         order_type: OrderType,
         reduce_only: bool = False,
         cloid: Optional[Cloid] = None,
+        builder: Optional[BuilderInfo] = None,
     ) -> Any:
         order: OrderRequest = {
-            "coin": coin,
+            "coin": name,
             "is_buy": is_buy,
             "sz": sz,
             "limit_px": limit_px,
@@ -111,15 +101,15 @@ class Exchange(API):
         }
         if cloid:
             order["cloid"] = cloid
-        return self.bulk_orders([order])
+        return self.bulk_orders([order], builder)
 
-    def bulk_orders(self, order_requests: List[OrderRequest]) -> Any:
+    def bulk_orders(self, order_requests: List[OrderRequest], builder: Optional[BuilderInfo] = None) -> Any:
         order_wires: List[OrderWire] = [
-            order_request_to_order_wire(order, self.coin_to_asset[order["coin"]]) for order in order_requests
+            order_request_to_order_wire(order, self.info.name_to_asset(order["coin"])) for order in order_requests
         ]
         timestamp = get_timestamp_ms()
 
-        order_action = order_wires_to_order_action(order_wires)
+        order_action = order_wires_to_order_action(order_wires, builder)
 
         signature = sign_l1_action(
             self.wallet,
@@ -138,7 +128,7 @@ class Exchange(API):
     def modify_order(
         self,
         oid: OidOrCloid,
-        coin: str,
+        name: str,
         is_buy: bool,
         sz: float,
         limit_px: float,
@@ -149,7 +139,7 @@ class Exchange(API):
         modify: ModifyRequest = {
             "oid": oid,
             "order": {
-                "coin": coin,
+                "coin": name,
                 "is_buy": is_buy,
                 "sz": sz,
                 "limit_px": limit_px,
@@ -165,7 +155,7 @@ class Exchange(API):
         modify_wires = [
             {
                 "oid": modify["oid"].to_raw() if isinstance(modify["oid"], Cloid) else modify["oid"],
-                "order": order_request_to_order_wire(modify["order"], self.coin_to_asset[modify["order"]["coin"]]),
+                "order": order_request_to_order_wire(modify["order"], self.info.name_to_asset(modify["order"]["coin"])),
             }
             for modify in modify_requests
         ]
@@ -191,17 +181,20 @@ class Exchange(API):
 
     def market_open(
         self,
-        coin: str,
+        name: str,
         is_buy: bool,
         sz: float,
         px: Optional[float] = None,
         slippage: float = DEFAULT_SLIPPAGE,
         cloid: Optional[Cloid] = None,
+        builder: Optional[BuilderInfo] = None,
     ) -> Any:
         # Get aggressive Market Price
-        px = self._slippage_price(coin, is_buy, slippage, px)
+        px = self._slippage_price(name, is_buy, slippage, px)
         # Market Order is an aggressive Limit Order IoC
-        return self.order(coin, is_buy, sz, px, order_type={"limit": {"tif": "Ioc"}}, reduce_only=False, cloid=cloid)
+        return self.order(
+            name, is_buy, sz, px, order_type={"limit": {"tif": "Ioc"}}, reduce_only=False, cloid=cloid, builder=builder
+        )
 
     def market_close(
         self,
@@ -210,6 +203,7 @@ class Exchange(API):
         px: Optional[float] = None,
         slippage: float = DEFAULT_SLIPPAGE,
         cloid: Optional[Cloid] = None,
+        builder: Optional[BuilderInfo] = None,
     ) -> Any:
         address = self.wallet.address
         if self.account_address:
@@ -228,13 +222,22 @@ class Exchange(API):
             # Get aggressive Market Price
             px = self._slippage_price(coin, is_buy, slippage, px)
             # Market Order is an aggressive Limit Order IoC
-            return self.order(coin, is_buy, sz, px, order_type={"limit": {"tif": "Ioc"}}, reduce_only=True, cloid=cloid)
+            return self.order(
+                coin,
+                is_buy,
+                sz,
+                px,
+                order_type={"limit": {"tif": "Ioc"}},
+                reduce_only=True,
+                cloid=cloid,
+                builder=builder,
+            )
 
-    def cancel(self, coin: str, oid: int) -> Any:
-        return self.bulk_cancel([{"coin": coin, "oid": oid}])
+    def cancel(self, name: str, oid: int) -> Any:
+        return self.bulk_cancel([{"coin": name, "oid": oid}])
 
-    def cancel_by_cloid(self, coin: str, cloid: Cloid) -> Any:
-        return self.bulk_cancel_by_cloid([{"coin": coin, "cloid": cloid}])
+    def cancel_by_cloid(self, name: str, cloid: Cloid) -> Any:
+        return self.bulk_cancel_by_cloid([{"coin": name, "cloid": cloid}])
 
     def bulk_cancel(self, cancel_requests: List[CancelRequest]) -> Any:
         timestamp = get_timestamp_ms()
@@ -242,7 +245,7 @@ class Exchange(API):
             "type": "cancel",
             "cancels": [
                 {
-                    "a": self.coin_to_asset[cancel["coin"]],
+                    "a": self.info.name_to_asset(cancel["coin"]),
                     "o": cancel["oid"],
                 }
                 for cancel in cancel_requests
@@ -269,7 +272,7 @@ class Exchange(API):
             "type": "cancelByCloid",
             "cancels": [
                 {
-                    "asset": self.coin_to_asset[cancel["coin"]],
+                    "asset": self.info.name_to_asset(cancel["coin"]),
                     "cloid": cancel["cloid"].to_raw(),
                 }
                 for cancel in cancel_requests
@@ -316,12 +319,11 @@ class Exchange(API):
             timestamp,
         )
 
-    def update_leverage(self, leverage: int, coin: str, is_cross: bool = True) -> Any:
+    def update_leverage(self, leverage: int, name: str, is_cross: bool = True) -> Any:
         timestamp = get_timestamp_ms()
-        asset = self.coin_to_asset[coin]
         update_leverage_action = {
             "type": "updateLeverage",
-            "asset": asset,
+            "asset": self.info.name_to_asset(name),
             "isCross": is_cross,
             "leverage": leverage,
         }
@@ -338,13 +340,12 @@ class Exchange(API):
             timestamp,
         )
 
-    def update_isolated_margin(self, amount: float, coin: str) -> Any:
+    def update_isolated_margin(self, amount: float, name: str) -> Any:
         timestamp = get_timestamp_ms()
-        asset = self.coin_to_asset[coin]
         amount = float_to_usd_int(amount)
         update_isolated_margin_action = {
             "type": "updateIsolatedMargin",
-            "asset": asset,
+            "asset": self.info.name_to_asset(name),
             "isBuy": True,
             "ntli": amount,
         }
@@ -399,6 +400,26 @@ class Exchange(API):
             timestamp,
         )
 
+    def usd_class_transfer(self, amount: float, to_perp: bool) -> Any:
+        timestamp = get_timestamp_ms()
+        str_amount = str(amount)
+        if self.vault_address:
+            str_amount += f" subaccount:{self.vault_address}"
+
+        action = {
+            "type": "usdClassTransfer",
+            "amount": str_amount,
+            "toPerp": to_perp,
+            "nonce": timestamp,
+        }
+        signature = sign_usd_class_transfer_action(self.wallet, action, self.base_url == MAINNET_API_URL)
+        return self._post_action(
+            action,
+            signature,
+            timestamp,
+        )
+
+    # Deprecated in favor of usd_class_transfer
     def user_spot_transfer(self, usdc: float, to_perp: bool) -> Any:
         usdc = int(round(usdc, 2) * 1e6)
         timestamp = get_timestamp_ms()
@@ -443,6 +464,22 @@ class Exchange(API):
             timestamp,
         )
 
+    def vault_usd_transfer(self, vault_address: str, is_deposit: bool, usd: int) -> Any:
+        timestamp = get_timestamp_ms()
+        vault_transfer_action = {
+            "type": "vaultTransfer",
+            "vaultAddress": vault_address,
+            "isDeposit": is_deposit,
+            "usd": usd,
+        }
+        is_mainnet = self.base_url == MAINNET_API_URL
+        signature = sign_l1_action(self.wallet, vault_transfer_action, None, timestamp, is_mainnet)
+        return self._post_action(
+            vault_transfer_action,
+            signature,
+            timestamp,
+        )
+
     def usd_transfer(self, amount: float, destination: str) -> Any:
         timestamp = get_timestamp_ms()
         action = {"destination": destination, "amount": str(amount), "time": timestamp, "type": "usdSend"}
@@ -454,6 +491,23 @@ class Exchange(API):
             timestamp,
         )
 
+    def spot_transfer(self, amount: float, destination: str, token: str) -> Any:
+        timestamp = get_timestamp_ms()
+        action = {
+            "destination": destination,
+            "amount": str(amount),
+            "token": token,
+            "time": timestamp,
+            "type": "spotSend",
+        }
+        is_mainnet = self.base_url == MAINNET_API_URL
+        signature = sign_spot_transfer_action(self.wallet, action, is_mainnet)
+        return self._post_action(
+            action,
+            signature,
+            timestamp,
+        )
+
     def withdraw_from_bridge(self, amount: float, destination: str) -> Any:
         timestamp = get_timestamp_ms()
         action = {"destination": destination, "amount": str(amount), "time": timestamp, "type": "withdraw3"}
@@ -488,3 +542,10 @@ class Exchange(API):
             ),
             agent_key,
         )
+
+    def approve_builder_fee(self, builder: str, max_fee_rate: str) -> Any:
+        timestamp = get_timestamp_ms()
+
+        action = {"maxFeeRate": max_fee_rate, "builder": builder, "nonce": timestamp, "type": "approveBuilderFee"}
+        signature = sign_approve_builder_fee(self.wallet, action, self.base_url == MAINNET_API_URL)
+        return self._post_action(action, signature, timestamp)
